#!/usr/bin/env python3

"""
Pytest tests

NOTE
$ coverage run -m pytest
$ coverage report ; coverage html
"""

from fastapi.testclient import TestClient
from opinion_miner_api.main import app

client = TestClient(app)


def test_predict_sentiment_positive():
    response = client.post("/predict", json={"text": "I love FastAPI!"})
    assert response.status_code == 200
    assert "result" in response.json()
    assert isinstance(response.json()["result"], dict)
    assert response.json()["result"]["label"] == "positive"


def test_predict_sentiment_negative():
    response = client.post("/predict", json={"text": "I hate bugs!"})
    assert response.status_code == 200
    assert "result" in response.json()
    assert isinstance(response.json()["result"], dict)
    assert response.json()["result"]["label"] == "negative"


def test_predict_sentiment_neutral():
    response = client.post("/predict", json={"text": "This is a statement."})
    assert response.status_code == 200
    assert "result" in response.json()
    assert isinstance(response.json()["result"], dict)
    assert response.json()["result"]["label"] == "neutral"


def test_predict_sentiment_missing_text():
    response = client.post("/predict", json={})
    assert response.status_code == 422  # Unprocessable Entity
    assert "detail" in response.json()

def test_predict_sentiment_empty_string():
    response = client.post("/predict", json={"text": ""})
    assert response.status_code == 200
    assert "result" in response.json()
    assert isinstance(response.json()["result"], dict)
    assert response.json()["result"]["label"] == "neutral"  # Assuming empty string is neutral

def test_predict_sentiment_special_characters():
    response = client.post("/predict", json={"text": "@#$%^&*()!"})
    assert response.status_code == 200
    assert "result" in response.json()
    assert isinstance(response.json()["result"], dict)
    assert response.json()["result"]["label"] == "neutral"  # Assuming special characters are neutral

def test_predict_sentiment_long_text():
    long_text = "I love FastAPI! " * 1000
    print("TEXT LENGTH: ", len(long_text))
    response = client.post("/predict", json={"text": long_text})
    assert response.status_code == 400
    assert "detail" in response.json()
    assert response.json()["detail"] == "Text too long. Maximum length is 1000 characters."

def test_predict_sentiment_invalid_json():
    response = client.post("/predict", content="Invalid JSON")
    assert response.status_code == 422  # Unprocessable Entity
    assert "detail" in response.json()
